const Counter = require('../models/counterModel');
const Token = require('../models/tokenModel');
const mongoose = require('mongoose');
const { ObjectId } = require('mongoose').Types;
exports.getAllCounters = async (req, res) => {
  try {
    const agg = [
      {
        '$lookup': {
          'from': 'users', 
          'localField': 'assignedUser', 
          'foreignField': '_id', 
          'as': 'result'
        }
      },
        {
          '$lookup': {
            'from': 'tokens', 
            'localField': 'currentToken', 
            'foreignField': '_id', 
            'as': 'value'
          }
        }
    ];

    const counters = await Counter.aggregate(agg);

    res.status(200).send(counters);
  } catch (error) {
    console.error('Error fetching counters:', error);
    res.status(500).json({ message: 'Internal Server Error' });
  }
};
exports.getCurrentToken = async (req, res) => {
  try {
    const { counterName } = req.body;
// console.log(counterName);
    const aggr = [
      {
        $match: {
          counterName: parseInt(counterName),
        }
      },
      {
        $lookup: {
          from: 'tokens',
          localField: 'currentToken',
          foreignField: '_id',
          as: 'tokenDetails'
        }
      },
    ];

    const counter = await Counter.aggregate(aggr);
    // console.log(counter);

    res.status(200).json(counter);
  } catch (error) {
    console.log(error);
    res.status(500).json({ message: 'Internal Server Error' });
  }
}

exports.nextToken = async (req, res) => {
  try {
    const { counterName, counterId, tokenId } = req.body;
    console.log(counterName,tokenId,counterId);
    await Counter.findByIdAndUpdate(counterId, { currentToken: null });
    await Token.findByIdAndUpdate(tokenId, { status: "Completed" });

    const counter = await Counter.findOne({ counterName: counterName });
console.log(counter,"counter find one")
    const nextToken = await Token.findOne({ selectedOption: counter.content, status: "Pending" });
console.log("find a token", nextToken)
    if (nextToken) {
      await Token.findByIdAndUpdate(nextToken._id, { counter: counter._id});
      await Counter.findByIdAndUpdate(counter._id, { currentToken: nextToken._id });
      console.log('Next token found:', nextToken.tokenNumber);
      res.status(200).json({message: "Next Token Found"});
    } else {
      res.status(200).json({message: "No Next Token Found"});
    }

  } catch (error) {
    console.log('Error:', error);
    res.status(500).json({ message: 'Internal Server Error' });
  }
}


// Create a new counter

exports.createCounter = async (req, res) => {
  const { counterName, content } = req.body;

  try {
    // Check if a counter with the same name already exists
    const existingCounter = await Counter.findOne({ counterName });
    if (existingCounter) {
      return res.status(400).json({ message: 'Counter name already exists. Please choose a different name.' });
    }

    // Create a new counter
    const counter = await Counter.create({
      counterName,
      content
    });

    // Aggregate to retrieve all data including counter and connected user
    const counterData = await Counter.aggregate([
      {
        $match: { _id: counter._id }
      },
      {
        $lookup: {
          from: 'users',
          localField: 'assignedUser',
          foreignField: '_id',
          as: 'assignedUserDetails'
        }
      }
    ]);

    res.status(201).json({ counterData });
  } catch (error) {
    console.error('Error creating counter:', error);
    res.status(500).json({ message: 'Internal Server Error' });
  }
};


// Get a specific counter by ID
exports.getCounterById = async (req, res) => {
  const counterId = req.params.id;

  try {
      const counter = await Counter.findById(counterId);
      res.status(200).json(counter);
  } catch (error) {
      res.status(500).json({ message: error.message });
  }
};


// Update a counter by ID
exports.updateCounterById = async (req, res) => {
  const counterId = req.params.id;
  const { counterName, content,active ,assignedUser} = req.body;

  try {
      const updatedCounter = await Counter.findByIdAndUpdate(
          counterId,
          { counterName, content,active,assignedUser },
          { new: true }
      );

      res.status(200).json(updatedCounter);
  } catch (error) {
      res.status(500).json({ message: error.message });
  }
};

// Delete a counter by ID
exports.deleteCounterById = async (req, res) => {
  const counterId = req.params.id;

  try {
      const deletCounter = await Counter.findByIdAndDelete(counterId);
      if(!deletCounter) {
          return res.status(404).json({ message: 'Counter not found' });
      }
      res.status(200).json({message: 'Counter deleted'});
  } catch (error) {
      res.status(500).json({ message: error.message });
  }
};

exports.getAllCounters2 = async (req, res) => {
  try {
    const counters = await Counter.find({assignedUser: null});
    res.status(200).send(counters);
  } catch (error) {
    console.error('Error fetching counters:', error);
    res.status(500).json({ message: 'Internal Server Error' });
  }
};

exports.getCounterByUserid = async (req, res) => {
  const counterId = req.params.id;

  try {
    // Ensure counterId is a valid ObjectId string
    const isValidObjectId = mongoose.Types.ObjectId.isValid(counterId);
    if (!isValidObjectId) {
      return res.status(400).json({ message: 'Invalid counter ID' });
    }

    const pipeline = [
      {
        $match: {
          assignedUser: new ObjectId(counterId)
        }
      },
      {
        $lookup: {
          from: "tokens",
          localField: "currentToken",
          foreignField: "_id",
          as: "tokenDetails"
        }
      }
    ];

    const result = await Counter.aggregate(pipeline);

    res.status(200).json(result);
  } catch (error) {
    console.error('Error:', error);
    res.status(500).json({ message: 'Internal Server Error' });
  }
};


