// token controler 
const Token = require("../models/tokenModel");
const Counter = require("../models/counterModel");

exports.generateToken = async (req, res) => {
  try {
    const { counterName, selectedOption } = req.body;

    // Find the current date
    const currentDate = new Date().toISOString().split("T")[0];

    // Find or create a counter
    let counter = await Counter.findOne({
      counterName: counterName,
      currentToken: null,
    });

    // check latest token today
    const latestToken = await Token.findOne({
      date: {
        $gte: new Date().setHours(0, 0, 0, 0),
        $lt: new Date().setHours(23, 59, 59, 999),
      },
    }).sort({ date: -1 });

    if (!counter) {
      const token = new Token({
        date: Date.now(),
        tokenNumber: latestToken ? latestToken.tokenNumber + 1 : 1,
        selectedOption: selectedOption,
      });
      await token.save();

      res.status(200).json({tokenNumber: token.tokenNumber});
    } else {
      const token = new Token({
        date: Date.now(),
        tokenNumber: latestToken ? latestToken.tokenNumber + 1 : 1,
        selectedOption: selectedOption,
        counter: counter._id,
      });
      await token.save();

      await Counter.findByIdAndUpdate(counter._id, {
        currentToken: token._id,
      });
      
      res.status(200).json({tokenNumber: token.tokenNumber});
    }
  } catch (error) {
    console.error("Error generating token:", error);
    res.status(500).json({ message: "Something went wrong" });
  }
};



exports.getDisplayData = async (req, res) => {
  try {
    const displayData = await Counter.aggregate([
      {
        $lookup: {
          from: 'tokens',
          localField: 'currentToken',
          foreignField: '_id',
          as: 'tokenDetails',
        },
      },
    ]);

    res.status(200).json(displayData);
  } catch (error) {
    console.error('Error fetching display data:', error);
    res.status(500).json({ message: 'Internal Server Error' });
  }
};

exports.getAllTokens = async (req, res) => {
  try {
    const agg = [
      {
        '$lookup': {
          'from': 'counters', 
          'localField': 'counter', 
          'foreignField': '_id', 
          'as': 'result'
        }
      }
    ];
    const tokens = await Token.aggregate(agg);
    res.status(200).json(tokens);
  } catch (error) {
    console.error('Error fetching tokens:', error);
    res.status(500).json({ message: 'Internal Server Error' });
  }
  
}