const userModel = require("../models/userModel");
const jwt = require("jsonwebtoken");
const bcrypt = require("bcrypt");
const Counter = require('../models/counterModel');
const mongoose = require('mongoose');
const { ObjectId } = require('mongodb');

exports.login = async (req, res) => {
  const { email, password } = req.body;

  try {
    // Validate inputs
    if (!email || !password) {
      return res.status(400).json({ error: "Email and password are required" });
    }

    // Find user by email
    const user = await userModel.findOne({ email });

    if (!user) {
      return res.status(404).json({ error: "User not found" });
    }

    // Compare passwords
    if (user.password !== password) {
      return res.status(401).json({ error: "Incorrect password" });
    }

    // Send success response
    res.json({
      message: "Success",
      role: user.role,
      id: user._id,
      assignedCounter: user.assignedCounter
    });
    
  } catch (error) {
    console.error(error);
    res.status(500).json({ error: "Login failed" });
  }
};

// Middleware to verify JWT token and check user role
exports.verifyToken = (req, res, next) => {
  const token = req.headers.authorization;

  if (!token) {
    return res
      .status(401)
      .json({ message: "Unauthorized - No token provided" });
  }

  jwt.verify(token, "your-secret-key", (err, decoded) => {
    if (err) {
      return res.status(401).json({ message: "Unauthorized - Invalid token" });
    }

    req.user = decoded;
    next();
  });
};



// Create a new user
exports.createUser = async (req, res) => {
  const { name, email, password, role, assignedCounter } = req.body;

  try {
    const assignedCounterObjectId = assignedCounter ? new ObjectId(assignedCounter) : null;
    
    // Check if the counter is available
    const counterAvailable = await Counter.findOne({ _id: assignedCounterObjectId, assignedUser: null });
    if (!counterAvailable) {
      return res.status(400).json({ message: 'Counter not available' });
    }
    
    // Create the user
    const user = await userModel.create({
      name,
      email,
      password,
      role,
      assignedCounter: assignedCounterObjectId
    });

    // Update the counter with the user's ID
    await Counter.findByIdAndUpdate(assignedCounterObjectId, { assignedUser: user._id });

    // Aggregate to retrieve all data including user and connected counter
    const userData = await userModel.aggregate([
      {
        $match: { _id: user._id }
      },
      {
        $lookup: {
          from: 'counters',
          localField: 'assignedCounter',
          foreignField: '_id',
          as: 'assignedCounterDetails'
        }
      }
    ]);

    // Sending user data to the counter
    await Counter.updateOne({ _id: assignedCounterObjectId }, { $push: { userDetails: userData } });

    res.status(201).json({ userData });
  } catch (error) {
    console.error('Error creating user:', error);
    res.status(500).json({ message: 'Internal Server Error' });
  }
};

// Get all users
// exports.getAllUsers = async (req, res) => {
//     try {
//         const aggr = [
//             {
//                 $lookup: {
//                     from: "counters",
//                     localField: "assignedCounter",
//                     foreignField: "_id",
//                     as: "counterdetails",
//                 }
//             },
//             {
//                 $unwind: {
//                     path: "$counterdetails",
//                 }
//             },
//             {
//                 $project: {
//                     _id: 1,
//                     name: 1,
//                     email: 1,
//                     role: 1,
//                     counterdetails: 1,
//                 }
//             }
//         ]

//         const users = await userModel.aggregate(aggr);
        
//         res.status(200).json(users);
//     } catch (error) {
//         res.status(500).json({ message: error.message });
//     }
// };

exports.getAllUsers=async(req,res)=>{
  try{
    const agg = [
      {
        '$lookup': {
          'from': 'counters', 
          'localField': 'assignedCounter', 
          'foreignField': '_id', 
          'as': 'result'
        }
      }
    ];
    
    const users=await userModel.aggregate(agg);
    
    res.status(200).json(users);
  }catch(error){
    res.status(500).json({message:error.message})
  }
}

// Get a specific user by ID
exports.getUserById = async (req, res) => {
  const userId = req.params.id; 
  try {
      const user = await userModel.findById(userId);
      res.status(200).send(user);
  } catch (error) {
      res.status(500).json({ message: error.message });
  }
};

// Update a user by ID
exports.updateUserById = async (req, res) => {
  const userId = req.params.id;
  const { assignedCounter,active } = req.body; 

  try {
      const updatedUser = await userModel.findByIdAndUpdate(
          userId,
          { assignedCounter ,active }, 
          { new: true }
      );

      res.status(200).json(updatedUser);
  } catch (error) {
      res.status(500).json({ message: error.message });
  }
};


// Delete a user by ID
exports.deleteUserById = async (req, res) => {
    const userId = req.params.id;

    try {
       const deleteUser = await userModel.findByIdAndDelete(userId);
       if (!deleteUser) {
           return res.status(404).json({ message: 'User not found' });
       }
        res.status(200).json({message: 'User deleted'});
    } catch (error) {
        res.status(500).json({ message: error.message });
    }
};